@@ -28,6 +28,7 @@ import static io.cucumber.gherkin.Locations.atColumn;
 import static io.cucumber.gherkin.Parser.Builder;
 import static io.cucumber.gherkin.Parser.RuleType;
 import static io.cucumber.gherkin.Parser.TokenType;
+import static java.util.Objects.requireNonNull;
 
 final class GherkinDocumentBuilder implements Builder<GherkinDocument> {
     private final List<Comment> comments = new ArrayList<>();
@@ -48,17 +49,17 @@ final class GherkinDocumentBuilder implements Builder<GherkinDocument> {
         stack.push(new AstNode(RuleType.None));
     }
 
-    private AstNode currentNode() {
-        return stack.peek();
+    private AstNode requiredCurrentNode() {
+        return requireNonNull(stack.peek());
     }
 
     @Override
     public void build(Token token) {
         if (token.matchedType == TokenType.Comment) {
-            Comment comment = new Comment(token.location, token.matchedText);
+            Comment comment = new Comment(token.location, token.getRequiredMatchedText());
             comments.add(comment);
         } else {
-            currentNode().add(token.matchedType.ruleType, token);
+            requiredCurrentNode().add(token.getRequiredMatchedType().ruleType, token);
         }
     }
 
@@ -71,66 +72,67 @@ final class GherkinDocumentBuilder implements Builder<GherkinDocument> {
     public void endRule(RuleType ruleType) {
         AstNode node = stack.pop();
         Object transformedNode = getTransformedNode(node);
-        currentNode().add(node.ruleType, transformedNode);
+        requiredCurrentNode().add(node.ruleType, transformedNode);
     }
 
     private Object getTransformedNode(AstNode node) {
-        switch (node.ruleType) {
-            case Step: {
+        return switch (node.ruleType) {
+            case Step -> {
                 Token stepLine = node.getToken(TokenType.StepLine);
-                return new Step(
+                yield new Step(
                         stepLine.location,
-                        stepLine.matchedKeyword,
+                        stepLine.getRequiredMatchedKeyword(),
                         stepLine.keywordType,
-                        stepLine.matchedText,
+                        stepLine.getRequiredMatchedText(),
                         node.getSingle(RuleType.DocString),
                         node.getSingle(RuleType.DataTable),
                         idGenerator.newId()
                 );
             }
-            case DocString: {
+            case DocString -> {
                 Token separatorToken = node.getTokens(TokenType.DocStringSeparator).get(0);
-                String mediaType = separatorToken.matchedText.isEmpty() ? null : separatorToken.matchedText;
+                String matchedText = separatorToken.getRequiredMatchedText();
+                String mediaType = matchedText.isEmpty() ? null : matchedText;
                 List<Token> lineTokens = node.getTokens(TokenType.Other);
                 String content = joinMatchedText(lineTokens, lineTokens.size());
-                return new DocString(
+                yield new DocString(
                         separatorToken.location,
                         mediaType,
                         content,
-                        separatorToken.matchedKeyword
+                        separatorToken.getRequiredMatchedKeyword()
                 );
             }
-            case DataTable: {
+            case DataTable -> {
                 List<TableRow> rows = getTableRows(node);
-                return new DataTable(rows.get(0).getLocation(), rows);
+                yield new DataTable(rows.get(0).getLocation(), rows);
             }
-            case Background: {
+            case Background -> {
                 Token backgroundLine = node.getToken(TokenType.BackgroundLine);
-                return new Background(
+                yield new Background(
                         backgroundLine.location,
-                        backgroundLine.matchedKeyword,
-                        backgroundLine.matchedText,
+                        backgroundLine.getRequiredMatchedKeyword(),
+                        backgroundLine.getRequiredMatchedText(),
                         getDescription(node),
                         getSteps(node),
                         idGenerator.newId()
                 );
             }
-            case ScenarioDefinition: {
+            case ScenarioDefinition -> {
                 AstNode scenarioNode = node.getRequiredSingle(RuleType.Scenario);
                 Token scenarioLine = scenarioNode.getToken(TokenType.ScenarioLine);
 
-                return new Scenario(
+                yield new Scenario(
                         scenarioLine.location,
                         getTags(node),
-                        scenarioLine.matchedKeyword,
-                        scenarioLine.matchedText,
+                        scenarioLine.getRequiredMatchedKeyword(),
+                        scenarioLine.getRequiredMatchedText(),
                         getDescription(scenarioNode),
                         getSteps(scenarioNode),
                         scenarioNode.getItems(RuleType.ExamplesDefinition),
                         idGenerator.newId()
                 );
             }
-            case ExamplesDefinition: {
+            case ExamplesDefinition -> {
                 AstNode examplesNode = node.getRequiredSingle(RuleType.Examples);
                 Token examplesLine = examplesNode.getToken(TokenType.ExamplesLine);
                 // rows is null when a Scenario Outline has no Examples table
@@ -145,30 +147,28 @@ final class GherkinDocumentBuilder implements Builder<GherkinDocument> {
                     tableBody = Collections.emptyList();
                 }
 
-                return new Examples(
+                yield new Examples(
                         examplesLine.location,
                         getTags(node),
-                        examplesLine.matchedKeyword,
-                        examplesLine.matchedText,
+                        examplesLine.getRequiredMatchedKeyword(),
+                        examplesLine.getRequiredMatchedText(),
                         getDescription(examplesNode),
                         tableHeader,
                         tableBody,
                         idGenerator.newId()
                 );
             }
-            case ExamplesTable: {
-                return getTableRows(node);
-            }
-            case Description: {
+            case ExamplesTable -> getTableRows(node);
+            case Description -> {
                 List<Token> lineTokens = node.getTokens(TokenType.Other);
                 // Trim trailing empty lines
                 int toIndex = lineTokens.size();
-                while (toIndex > 0 && lineTokens.get(toIndex - 1).line.isEmpty()) {
+                while (toIndex > 0 && lineTokens.get(toIndex - 1).getRequiredLine().isEmpty()) {
                     toIndex--;
                 }
-                return joinMatchedText(lineTokens, toIndex);
+                yield joinMatchedText(lineTokens, toIndex);
             }
-            case Feature: {
+            case Feature -> {
                 AstNode header = node.getRequiredSingle(RuleType.FeatureHeader);
                 List<Tag> tags = getTags(header);
                 Token featureLine = header.getToken(TokenType.FeatureLine);
@@ -186,17 +186,17 @@ final class GherkinDocumentBuilder implements Builder<GherkinDocument> {
                     children.add(new FeatureChild(rule, null, null));
                 }
 
-                return new Feature(
+                yield new Feature(
                         featureLine.location,
                         tags,
-                        featureLine.matchedLanguage,
-                        featureLine.matchedKeyword,
-                        featureLine.matchedText,
+                        featureLine.getRequiredMatchedLanguage(),
+                        featureLine.getRequiredMatchedKeyword(),
+                        featureLine.getRequiredMatchedText(),
                         getDescription(header),
                         children
                 );
             }
-            case Rule: {
+            case Rule -> {
                 AstNode header = node.getRequiredSingle(RuleType.RuleHeader);
                 Token ruleLine = header.getToken(TokenType.RuleLine);
 
@@ -213,45 +213,47 @@ final class GherkinDocumentBuilder implements Builder<GherkinDocument> {
                     children.add(new RuleChild(null, scenario));
                 }
 
-                return new Rule(
+                yield new Rule(
                         ruleLine.location,
                         tags,
-                        ruleLine.matchedKeyword,
-                        ruleLine.matchedText,
+                        ruleLine.getRequiredMatchedKeyword(),
+                        ruleLine.getRequiredMatchedText(),
                         getDescription(header),
                         children,
                         idGenerator.newId()
                 );
-
             }
-            case GherkinDocument: {
+            case GherkinDocument -> {
                 Feature feature = node.getSingle(RuleType.Feature);
                 // feature is null when the file is empty or contains only comments/whitespace, or no Cucumber feature
-                return new GherkinDocument(uri, feature, comments);
+                yield new GherkinDocument(uri, feature, comments);
             }
-
-        }
-        return node;
+            // Do nothing.
+            default -> node;
+        };
     }
 
     private static String joinMatchedText(List<Token> lineTokens, int toIndex) {
         StringBuilder content = new StringBuilder(FEATURE_FILE_AVERAGE_LINE_LENGTH * lineTokens.size());
         for (int i = 0; i < toIndex; i++) {
             Token lineToken = lineTokens.get(i);
-            content.append(lineToken.matchedText).append('\n');
+            String matchedText = lineToken.getRequiredMatchedText();
+            content.append(matchedText).append('\n');
         }
         int contentLength = content.length();
         if (contentLength > 0) {
-            content.setLength(contentLength - 1); // Remove the last \n
+            // Remove the last \n
+            content.setLength(contentLength - 1); 
         }
         return content.toString();
     }
 
-    @SuppressWarnings("ForLoopReplaceableByForEach") // classic 'for' loop is ~2x faster than 'for-each'
+    @SuppressWarnings("ForLoopReplaceableByForEach")
     private List<TableRow> getTableRows(AstNode node) {
         List<Token> tokens = node.getTokens(TokenType.TableRow);
         int tokenSize = tokens.size();
         List<TableRow> rows = new ArrayList<>(tokenSize);
+        // classic 'for' loop is ~2x faster than 'for-each'
         for (int i = 0; i < tokenSize; i++) {
             Token token = tokens.get(i);
             rows.add(new TableRow(token.location, getCells(token), idGenerator.newId()));
@@ -260,12 +262,13 @@ final class GherkinDocumentBuilder implements Builder<GherkinDocument> {
         return rows;
     }
 
-    @SuppressWarnings("ForLoopReplaceableByForEach") // classic 'for' loop is ~2x faster than 'for-each'
+    @SuppressWarnings("ForLoopReplaceableByForEach")
     private void ensureCellCount(List<TableRow> rows) {
         if (rows.isEmpty())
             return;
 
         int firstRowCellsSize = rows.get(0).getCells().size();
+        // classic 'for' loop is ~2x faster than 'for-each'
         for (int i = 0, rowsSize = rows.size(); i < rowsSize; i++) {
             TableRow row = rows.get(i);
             if (row.getCells().size() != firstRowCellsSize) {
@@ -274,11 +277,15 @@ final class GherkinDocumentBuilder implements Builder<GherkinDocument> {
         }
     }
 
-    @SuppressWarnings("ForLoopReplaceableByForEach") // classic 'for' loop is ~2x faster than 'for-each'
+    @SuppressWarnings("ForLoopReplaceableByForEach") 
     private List<TableCell> getCells(Token token) {
         List<LineSpan> matchedItems = token.matchedItems;
+        if (matchedItems == null) {
+            return new ArrayList<>(0);
+        }
         int itemSize = matchedItems.size();
         List<TableCell> cells = new ArrayList<>(itemSize);
+        // classic 'for' loop is ~2x faster than 'for-each'
         for (int i = 0; i < itemSize; i++) {
             LineSpan cellItem = matchedItems.get(i);
             TableCell tableCell = new TableCell(
@@ -300,15 +307,19 @@ final class GherkinDocumentBuilder implements Builder<GherkinDocument> {
 
     private List<Tag> getTags(AstNode node) {
         AstNode tagsNode = node.getSingle(RuleType.Tags);
-        if (tagsNode == null) {// tags are optional
-            return Collections.emptyList();
+        // tags are optional
+        if (tagsNode == null) {
+            return new ArrayList<>(0);
         }
 
         List<Token> tokens = tagsNode.getTokens(TokenType.TagLine);
         List<Tag> tags = new ArrayList<>();
         for (Token token : tokens) {
-            for (LineSpan tagItem : token.matchedItems) {
-                tags.add(new Tag(atColumn(token.location, tagItem.column), tagItem.text, idGenerator.newId()));
+            List<LineSpan> matchedItems = token.matchedItems;
+            if (matchedItems != null) {
+                for (LineSpan tagItem : matchedItems) {
+                    tags.add(new Tag(atColumn(token.location, tagItem.column), tagItem.text, idGenerator.newId()));
+                }
             }
         }
         return tags;
@@ -316,7 +327,7 @@ final class GherkinDocumentBuilder implements Builder<GherkinDocument> {
 
     @Override
     public GherkinDocument getResult() {
-        return currentNode().getSingle(RuleType.GherkinDocument);
+        return requiredCurrentNode().getRequiredSingle(RuleType.GherkinDocument);
     }
 
 }
